@@ -484,8 +484,10 @@ class ProblemService:
 
     @staticmethod
     def create_problem(folder, problem_name, problem_code=None, lang="vi", translations=[], overwrite=False,
-                       tags=[], difficulty=0.0, programming_language='py', gen_sample=False):
+                       tags=[], difficulty=0.0, programming_language='py', gen_sample=False, testcase_format='ucode'):
         problem = Problem()
+        if testcase_format:
+            problem.testcase_format = testcase_format
         problem.name = problem_name
         if not problem_code:
             problem_code = problem.name
@@ -564,6 +566,10 @@ class ProblemService:
             if constraints_i:
                 problem.constraints = join_lines(constraints_c[constraints_i[0]])
 
+            subtasks_i, subtasks_c = find_section('(#+\s*Subtasks.*|Subtask\s*$|#+\s*Sub tasks.*)', lines)
+            if subtasks_i:
+                problem.constraints += "\n\nSubtasks:\n\n" + join_lines(subtasks_c[subtasks_i[0]])
+
             sample_input_i, sample_input_c = find_section('#+\s*Sample input(.*)', lines)
             sample_output_i, sample_output_c = find_section('#+\s*Sample (ou|Ou|out|Out)put(.*)', lines)
             explanation_i, explanation_c = find_section('#+\s*Explanation(.*)', lines)
@@ -845,10 +851,10 @@ class ProblemService:
                     CLog.warn(f'`{testcase_file}` file not existed, skipping...')
                 else:
                     CLog.info(f'`{testcase_file}` file FOUND, reading testcases...')
-                    tests = TestcaseService.load_testcases(testcase_file, format='ucode')
+                    tests = TestcaseService.load_testcases(testcase_file)
                     # print(f"reading testcases in file {testcase_file}: {len(tests)}")
                     for t in tests:
-                        _input, _output = t['input'], t['output']
+                        _input, _output = t.input, t.output
                         # print(f"Input: {input}, output: {output}")
                         if _input not in input_set:
                             problem.testcases.append(TestCase(input=_input, output=_output))
@@ -921,7 +927,7 @@ class ProblemService:
                 problem.statement_language = lang
             print("src", problem.src_url)
 
-            problem.testcase_format = "ucode"
+            # problem.testcase_format = "ucode"
 
             meta_dict = {
               "code": problem.code,
@@ -1027,9 +1033,9 @@ class ProblemService:
 
         if save_testcases:
             from ucode.services.testcase.testcase_service import TestcaseService
-            TestcaseService.save_testcases(problem_folder, problem, format="ucode")
-            if save_testcases_only:
-                CLog.warn("DON'T FORGET to change the 'testcase_format' to 'ucode' in meta data file.")
+            TestcaseService.save_testcases(problem_folder, problem, format=problem.testcase_format)
+            # if save_testcases_only:
+            #     CLog.warn("DON'T FORGET to change the 'testcase_format' to 'ucode' in meta data file.")
 
         problem_folder = os.path.abspath(problem_folder)
 
@@ -1048,6 +1054,7 @@ class ProblemService:
 
     @staticmethod
     def load_testcases(problem_folder, format: str, problem: Problem):
+        CLog.info(f"Loading testcases with specified format {format} from {problem_folder}")
         if format == ProblemFolderFormat.ucode.value:
             ProblemService.load_ucode_testcases(problem_folder, problem)
         elif format == ProblemFolderFormat.themis.value:
@@ -1069,7 +1076,7 @@ class ProblemService:
             if not os.path.exists(testcase_file) or not os.path.isfile(testcase_file):
                 CLog.info(f'`{testcase_file}` file not existed, skipping...')
             else:
-                tests = TestcaseService.load_testcases(testcase_file, format='ucode')
+                tests = TestcaseService.load_testcases(testcase_file)
                 print(f"reading testcases in file {testcase_file}: {len(tests)}")
                 for t in tests:
                     _input, _output, _name = t['input'], t['output'], t['name']
@@ -1087,7 +1094,20 @@ class ProblemService:
 
     @staticmethod
     def load_themis_testcases(problem_folder, problem: Problem):
-        problem.testcases = TestcaseService.load_testcases(problem_folder)
+        # uu tien load file testcases.zip truoc
+        testcase_zip = os.path.join(problem_folder, "testcases.zip")
+        if os.path.exists(testcase_zip):
+            CLog.info(f"Found testcases.zip file: {testcase_zip}")
+            problem.testcases = TestcaseService.load_testcases(testcase_zip)
+        else:
+            # uu tien file .zip neu co 1 file ton tai
+            testcase_zips = glob.glob(os.path.join(problem_folder, "*.zip"))
+            if len(testcase_zips) == 1:
+                testcase_zip = testcase_zips[0]
+                CLog.info(f"Found testcases.zip file: {testcase_zip}")
+                problem.testcases = TestcaseService.load_testcases(testcase_zip)
+            else:
+                problem.testcases = TestcaseService.load_testcases(problem_folder)
 
     @staticmethod
     def load_cms_testcases(problem_folder, problem: Problem):
