@@ -29,7 +29,7 @@ class TestcaseService:
                         .zip file if format is 'cms',
                         folder if format is 'themis'
         @param format: ucode / cms / themis / hackerrank
-        @return:
+        @return: List[Testcase]
         """
 
         path = os.path.abspath(path)
@@ -85,7 +85,7 @@ class TestcaseService:
 
     @staticmethod
     def get_testcase_files_list_from_folder(path):
-        all_files = glob.glob(os.path.join(path, "*"))
+        all_files = sorted(glob.glob(os.path.join(path, "*")))
         folders = [f for f in all_files if os.path.isdir(f)]
         print("Folder", folders)
         has_input = has_output = False
@@ -118,7 +118,7 @@ class TestcaseService:
 
     @staticmethod
     def get_testcase_files_list_from_zip(path, zip_file):
-        all_files = zip_file.namelist()
+        all_files = sorted(zip_file.namelist())
         has_input = has_output = False
         folder_count = 0
         # prefix =
@@ -225,8 +225,8 @@ class TestcaseService:
             for (input_file, output_file) in inside_archive_files:
                 fi = myzip.open(input_file)
                 fo = myzip.open(output_file)
-                _input = fi.read().decode("utf-8")
-                _output = fo.read().decode("utf-8")
+                _input = TestcaseService.read_file(input_file, fi, myzip)
+                _output = TestcaseService.read_file(output_file, fo, myzip)
                 testcases.append(TestCase(input=_input, output=_output))
 
         CLog.info(f"DONE: {len(testcases)} testcases loaded!")
@@ -254,9 +254,9 @@ class TestcaseService:
             output_file = output_files[0]
 
             # print(input_file, output_file)
-            with open(input_file, 'r') as fi:
+            with open(input_file, 'r', encoding="utf-8") as fi:
                 _input = fi.read()
-            with open(output_file, 'r') as fo:
+            with open(output_file, 'r', encoding="utf-8") as fo:
                 _output = fo.read()
             testcases.append(TestCase(input=_input, output=_output))
         return testcases
@@ -282,7 +282,8 @@ class TestcaseService:
             if line.startswith("###"):
 
                 if count > 0:
-                    testcases.append({'input': inputi.rstrip(), 'output': outputi.rstrip(), 'name': name})
+                    # testcases.append({'input': inputi.rstrip(), 'output': outputi.rstrip(), 'name': name})
+                    testcases.append(TestCase(input=inputi.rstrip(), output=outputi.rstrip(), name=name))
 
                 name = line[3:].strip()
                 # print("N:", name)
@@ -301,7 +302,8 @@ class TestcaseService:
                 else:
                     inputi += line
         if inputi.rstrip() or outputi.rstrip():
-            testcases.append({'input': inputi.rstrip(), 'output': outputi.rstrip(), 'name': name})
+            # testcases.append({'input': inputi.rstrip(), 'output': outputi.rstrip(), 'name': name})
+            testcases.append(TestCase(input=inputi.rstrip(), output=outputi.rstrip(), name=name))
 
         return testcases
 
@@ -353,7 +355,7 @@ class TestcaseService:
         return output, compile_time, execute_time
 
     @staticmethod
-    def save_testcases(problem_folder, problem: Problem, format="both"):
+    def save_testcases(problem_folder, problem: Problem, format="ucode"):
         """
 
         @param problem_folder:
@@ -363,54 +365,79 @@ class TestcaseService:
         """
         template_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), '../dsa/templates')
 
-        if format:
-            if format == "ucode" or format == "both":
-                CLog.info("Writing testcases in ucode format")
-                testcases = problem.testcases
-                if testcases:
-                    with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
-                        template = Template(file_.read())
-                        for i, t in enumerate(testcases):
-                            t.name = "Test #%02d" % (i + 1)
-                        content = template.render(testcases=testcases)
-                        testcase_path = os.path.join(problem_folder, "testcases.txt")
-                        f = open(testcase_path, 'w', encoding="utf-8", newline='')
-                        f.write(dos2unix(content))
-                        f.close()
-                        CLog.info(f"Testcases saved to '{testcase_path}'")
+        if format == "ucode":
+            CLog.info("Writing testcases in ucode format")
+            testcases = problem.testcases
+            if testcases:
+                with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
+                    template = Template(file_.read())
+                    for i, t in enumerate(testcases):
+                        t.name = "Test #%02d" % (i + 1)
+                    content = template.render(testcases=testcases)
+                    testcase_path = os.path.join(problem_folder, "testcases.txt")
+                    f = open(testcase_path, 'w', encoding="utf-8", newline='')
+                    f.write(dos2unix(content))
+                    f.close()
+                    CLog.info(f"Testcases saved to '{testcase_path}'")
 
-                if problem.stock_testcases:
-                    with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
-                        template = Template(file_.read())
-                        for i, t in enumerate(problem.stock_testcases):
-                            t.name = "Test #%02d" % (i + 1)
-                        content = template.render(testcases=problem.stock_testcases)
-                        testcase_path = os.path.join(problem_folder, "testcases_stock.txt")
-                        f = open(testcase_path, 'w', encoding="utf-8", newline='')
-                        f.write(dos2unix(content))
-                        f.close()
-                        CLog.info(f"Testcases saved to '{testcase_path}'")
+            if problem.stock_testcases:
+                with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
+                    template = Template(file_.read())
+                    for i, t in enumerate(problem.stock_testcases):
+                        t.name = "Test #%02d" % (i + 1)
+                    content = template.render(testcases=problem.stock_testcases)
+                    testcase_path = os.path.join(problem_folder, "testcases_stock.txt")
+                    f = open(testcase_path, 'w', encoding="utf-8", newline='')
+                    f.write(dos2unix(content))
+                    f.close()
+                    CLog.info(f"Testcases saved to '{testcase_path}'")
 
+            if problem.testcases_sample:
+                with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
+                    template = Template(file_.read())
+                    for i, t in enumerate(problem.testcases_sample):
+                        t.name = "Test #%02d" % (i + 1)
+                    content = template.render(testcases=problem.testcases_sample)
+                    testcase_path = os.path.join(problem_folder, "testcases_sample.txt")
+                    f = open(testcase_path, 'w', encoding="utf-8", newline='')
+                    f.write(dos2unix(content))
+                    f.close()
+        elif format == "themis":
+            CLog.info("Writing testcases in themis format")
+            with ZipFile(os.path.join(problem_folder, 'testcases.zip'), 'w') as zip:
                 if problem.testcases_sample:
-                    with open(os.path.join(template_path, 'testcases.txt.j2'), encoding="utf-8") as file_:
-                        template = Template(file_.read())
-                        for i, t in enumerate(problem.testcases_sample):
-                            t.name = "Test #%02d" % (i + 1)
-                        content = template.render(testcases=problem.testcases_sample)
-                        testcase_path = os.path.join(problem_folder, "testcases_sample.txt")
-                        f = open(testcase_path, 'w', encoding="utf-8", newline='')
-                        f.write(dos2unix(content))
-                        f.close()
+                    for i, t in enumerate(problem.testcases):
+                        folder = "SampleTest%02d" % (i + 1)
+                        zip.writestr(os.path.join(folder, 'DATA.INP'), dos2unix(t.input))
+                        zip.writestr(os.path.join(folder, 'DATA.OUT'), dos2unix(t.output))
+                testcases = []
+                if problem.stock_testcases:
+                    testcases += problem.stock_testcases
+                if problem.testcases:
+                    testcases += problem.testcases
+                if testcases:
+                    for i, t in enumerate(testcases):
+                        folder = "Test%02d" % (i + 1)
+                        zip.writestr(os.path.join(folder, 'DATA.INP'), dos2unix(t.input))
+                        zip.writestr(os.path.join(folder, 'DATA.OUT'), dos2unix(t.output))
+                        # os.makedirs(folder, exist_ok=True)
+                        # folder = os.path.join(problem_folder, "Test%02d" % (i + 1))
+                        # with open(os.path.join(folder, 'DATA.INP'), 'w', encoding="utf-8") as fi:
+                        #     fi.write(dos2unix(t.input))
+                        # with open(os.path.join(folder, 'DATA.OUT'), 'w', encoding="utf-8") as fo:
+                        #     fo.write(dos2unix(t.output))
+
+        else:
+            CLog.error(f"Saving testcases to format `{format}` not supported yet.")
 
     @staticmethod
     def generate_testcases(problem_folder, testcase_count=20,
-                           programming_language=None, format=None, overwrite=False):
+                           programming_language=None, overwrite=False):
         """
 
         @param problem_folder:
         @param testcase_count:
         @param programming_language: py | cpp | pas, None = auto detect
-        @param format: ucode | themis | both | None, None --> not save to disk
         @return:
         """
         from ucode.services.dsa.problem_service import ProblemService
@@ -461,15 +488,32 @@ class TestcaseService:
 
             testcases.append(TestCase(input=_input, output=_output))
             print("Testcase #%02d generated! Input generation compile time: %.2fs, "
-                  "input generation running time: %.2fs, solution complie time: %.2f, solution running time: %.2fs"
+                  "input generation running time: %.2fs, solution compile time: %.2f, solution running time: %.2fs"
                   % (i, input_compile_time, input_time, compile_time, output_time))
 
-        if format:
+        if meta['testcase_format']:
             problem = Problem()
             problem.testcases = testcases
-            TestcaseService.save_testcases(problem_folder, problem, format)
+            TestcaseService.save_testcases(problem_folder, problem, meta['testcase_format'])
         return testcases
 
+    @staticmethod
+    def read_file(file_path, file, file_reader=None, ):
+        try:
+            data = file.read().decode("utf-8")
+            return data
+        except UnicodeDecodeError as exception:
+            if file_reader:
+                file = file_reader.open(file_path)
+            else:
+                file = open(file_path, 'r')
+            file.seek(exception.start)
+            exception.reason = f'File {file.name} corrupted at: {file.read()[:100]}...'
+            raise exception
+        except Exception as exception:
+            # todo unknown exception
+            raise exception
+
 
 if __name__ == "__main__":
     # folder = "D:\\projects\\ucode\\dsa-thematic-problems\\de_thi_hsg\\hanoi_2020_2021_thcs\\p4_itable"
@@ -491,8 +535,11 @@ if __name__ == "__main__":
     # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\themis\sample_testcases_nested_dif_case.zip"
     # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\hackerrank\testcases_hackerrank.zip"
     # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\hackerrank\testcases_hackerrank_nested.zip"
+    # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\cms"
     # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\cms\BANK.zip"
     # testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\cms\cms_sample_nested.zip"
+
+    testcase_file = r"D:\projects\ucode\ucode-cli\ucode\services\testcase\sample_testcases\bridge.zip"
     testcases = TestcaseService.load_testcases(testcase_file)
     print(testcases)
     print(len(testcases))
