@@ -187,13 +187,31 @@ class UCode:
                 "status": status,
                 "visibility": visibility,
                 "ucoin": ucoin,
-                "source": problem.src_url if problem.src_url else "",
+                # "source": problem.src_url if problem.src_url else "",
+                "source_detail": problem.src_url if problem.src_url else "",
                 'tags': problem.tags
             }
+            if problem.editorial:
+                data['hint'] = json.dumps({
+                    "text": problem.editorial,
+                    "text_type": "markdown"
+                })
+            if problem.solutions:
+                solution_text = ""
+                for solution in problem.solutions:
+                    solution_text += f"```{solution['lang']}\n"
+                    solution_text += f"{solution['code']}\n"
+                    solution_text += f"```\n\n\n"
+
+                data['solutions'] = json.dumps({
+                    "text": solution_text,
+                    "text_type": "markdown"
+                })
+
             if request_approval:
                 data['approval_status'] = 'waiting_for_approval'
-            if problem.solution:
-                data["solution"] = problem.solution
+            # if problem.solution:
+            #     data["solution"] = problem.solution
         elif isinstance(problem, Question):
             question: Question = problem
             data = {
@@ -323,7 +341,7 @@ class UCode:
 
         return question_ids
 
-    def upload_testcases(self, problem_id, testcases: List[TestCase], score=10, sample_score=10):
+    def upload_testcases(self, problem_id, testcases: List[TestCase], score=10, sample_score=0):
         url = self.get_api_url(f"/question/{problem_id}/add-testcases")
         testcase_data = []
         for i, testcase in enumerate(testcases):
@@ -548,26 +566,32 @@ def create_problems_to_lesson_ucode(env="dev", domain="ucode.vn"):
 
     # base_folder = r"D:\projects\ucode\weekly-algorithm-problems\competitions\teko-hcm"
     # base_folder = r"D:\projects\freecontest\mock_tests\mock01"
-    base_folder = r"D:\projects\ucode\competitive_programing_topics\tests\sample_tests\thcs_testTHT"
+    # base_folder = r"D:\projects\ucode\competitive_programing_topics\tests\sample_tests\thcs_testTHT"
+    # base_folder = r"D:\projects\ucode\dsa-problem-bank\tests_lamcherry (9-2021)\thcs_tests\test_02"
+
     # base_folder = "D:\\projects\\ucode\\competitive_programing_topics\\Data_Structures\\Heap_PriorityQueue"
-    problems = ProblemService.read_all_problems(base_folder, nested_folder=False)
+    # problems = ProblemService.read_all_problems(base_folder, nested_folder=False)
     # problems = ProblemService.read_all_problems_v1(base_folder, nested_folder=False)
 
-    # prob_folder = "D:\\projects\\freecontest\\formatted\\Free Contest 80\\groups"
-    # problems = [(prob_folder, ProblemService.load(prob_folder))]
+    prob_folder = r"D:\projects\ucode\dsa-problem-bank\lamcherry\2021-09\thpt_tests\test_05\p04_bookshelves"
+    problems = [(prob_folder, ProblemService.load(prob_folder))]
 
-    lesson_id = 16614
+    lesson_id = 22910
     ucode = UCode(env=env, domain=domain)
     #
 
     for i, (problem_folder, problem) in enumerate(problems):
         problem = ProblemService.load(problem_folder, load_testcase=True)
         # problem = ProblemService.load_v1(problem_folder, load_testcase=True, translations=['vi'])
-        print(i + 1, problem.name, "testcases:", len(problem.testcases))
+        print(i + 1, problem.name, "testcases:", len(problem.testcases), ", sample test:", len(problem.testcases_sample))
+        print("Solution:", json.dumps(problem.solutions))
+        print("Editorial:", problem.editorial)
+        print("Constraint: ", problem.constraints)
+
         prob_id = ucode.create_problem(problem=problem, xp=-1, score=100)
         CLog.info(f"Created problem: {prob_id}")
-        ucode.add_question_to_quiz(quiz_id=lesson_id, question_ids=[prob_id])
-        print(f"Problem uploaded with id #{prob_id} and added to quiz {lesson_id}")
+        # ucode.add_question_to_quiz(quiz_id=lesson_id, question_ids=[prob_id])
+        # print(f"Problem uploaded with id #{prob_id} and added to quiz {lesson_id}")
 
     print("Total problems:", len(problems))
 
@@ -794,7 +818,7 @@ if __name__ == "__main__":
     # # prepare_testcases("/home/thuc/projects/ucode/problemtools/problems/cs/Arcade_TheCore/p001_Intro_Gates__addTwoDigits")
     # hr.upload_testcases(163975, "/home/thuc/projects/ucode/problemtools/problems/cs/Arcade_TheCore/p001_Intro_Gates__addTwoDigits/testcases_hackerrank.zip")
     # # hr_prob = hr.create_problem(problem)
-    # # print(json.dumps(hr_prob))
+    # # print(json.dumps(hr_prob))g
     #
     # "https://www.hackerrank.com/rest/administration/contests/46715/challenge?slug=add-two-digits&track_id=0&chapter_id=0&weight=100"
     #
